package vishnu.testvis.dendro;

import java.util.*;

class Iterative
{
    Vector vector;
    int VECTOR_SIZE;
    
    Iterative(int vsize)
    {
	VECTOR_SIZE = vsize;
    }
    
    public void printVector(double[] a)
    {
        int i;

	for (i = 0; i < VECTOR_SIZE; i++)
    	    System.out.println("v[" + i + "] = " + a[i]);
    }
    
    
    void cluster_center (Cluster cluster, double [][] vectors, double[] centroid)
    {
        int i;
	
      	if (cluster._id == -1)
	    {
		cluster_center (cluster._child_1, vectors, centroid);
		cluster_center (cluster._child_2, vectors, centroid);
	    }
        else
	    {
		double[] doubleArray = vectors[cluster._id];
		for (i = 0; i < doubleArray.length; i++)
	            centroid[i] += doubleArray[i];
	    }
    }
    
    void compute_cluster_centers (ClusterArray clusters, int num_of_clusters, double [][] vectors)
    {
	int i, j; double ssum;
	Cluster clust;
	
	for (i = 0; i < num_of_clusters; i++)
	    {
		clust = (Cluster)clusters.contents.elementAt(i);
	        clust._centroid = new double[VECTOR_SIZE];
		cluster_center (clust, vectors, clust._centroid);
		for (ssum = j = 0; j < VECTOR_SIZE; j++)
		    ssum += clust._centroid[j] * clust._centroid[j];
		if(ssum>0)                // normalise
		    for(ssum = 1.0/Math.sqrt(ssum), j = 0; j < VECTOR_SIZE; j++)
			clust._centroid[j] *= ssum;
	    }
    }
    

    void place_item (ClusterArray clusters,
		     int num_of_clusters,
		     double [][] vectors,
		     int item_id,  Distance sim_func)
    {
	double max_sim, sim;
	int max_sim_index;
	
	int i;
	
	max_sim_index = 0;
        Cluster clust = (Cluster)clusters.contents.elementAt(0);
	max_sim = sim_func.get(clust._centroid,
			       vectors[item_id], VECTOR_SIZE);
	
  	    for (i = 1; i < num_of_clusters; i++)
    	{
            clust = (Cluster)clusters.contents.elementAt(i);
      	    sim = sim_func.get(clust._centroid,
			vectors[item_id], VECTOR_SIZE);

            if (sim < max_sim)
	        {
	  	        max_sim_index = i;
	  	        max_sim = sim;
	        }
    	}
  	    
  	    clust = (Cluster)clusters.contents.elementAt(max_sim_index);
        clust._iter_items.addElement(new TheInt(item_id));
	    clust._iter_items_num++;
    }

    // add all documents not yet added to the existing cluster hierarchy (which is based on the 
    // limit first documents. Hence there are vector.length - limit docs to add
    // consequently index runs from limit (start_index) to vector.length (end_index)
    void clustering (ClusterArray clusters, TheInt num_of_clusters,
                      double[][] vectors, int start_index, int end_index,
                      Distance sim_func)
    {
        int i;
    
  	    if (start_index >= end_index)
    	    	return;

  	    compute_cluster_centers(clusters, num_of_clusters.val, vectors);

  	    for (i = start_index; i < end_index; i++)
      	    	place_item(clusters, num_of_clusters.val, vectors, i, sim_func);
    }

}
