/**
 *#########################################################################
 *
 * A component of the Administration Tool (GAI) application, part of the 
 * Greenstone digital library suite from the New Zealand Digital Library 
 * Project at the University of Waikato, New Zealand.
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: Chi-Yu Huang, Greenstone Digital Library, University of Waikato
 *
 *########################################################################
 */

package org.greenstone.admin;

import org.greenstone.admin.GAI;
import org.greenstone.admin.gui.TabbedPane;
import org.greenstone.admin.gui.MenuBar;
import org.greenstone.admin.gui.LogPane;
import org.greenstone.admin.gui.ConfPane;

import org.greenstone.core.Configuration;
import org.greenstone.core.Dictionary;
import org.greenstone.core.DebugStream;
import org.greenstone.core.util.Utility;

//import java AWT classes
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;

public class GAIManager 
    extends JFrame 
    implements ActionListener, ChangeListener{
    
    //private static final Dimension SIZE = new Dimension (800, 600);
    private Dimension size = null;
    //public static String gsdl3_path = null;
    
    private JTabbedPane tab_pane = null;

    // Call the ConfPane
    public ConfPane conf_pane = null;
    
    public JPanel ext_pane = null;
       
    public JPanel monitor_pane = null;
    
    // Call the LogPane
    public LogPane log_pane = null;
    
    static public ImageIcon CONF_ICON = null; 
    static public ImageIcon EXT_ICON = null; 
    static public ImageIcon MONITOR_ICON = null; 
    static public ImageIcon LOG_ICON = null; 
    
    // Create a menu bar in the main Pane
    public MenuBar menu_bar = null;
    // Setting up the MenuItem
    public JMenu menu = null;
    
    private JPanel previous_pane = null;
    private JPanel content_pane = null;
    //public static String admin_path = null;
    
    //Contructor
    public GAIManager(Dimension size){
	super();
	this.size = size;
	this.setDefaultCloseOperation(EXIT_ON_CLOSE);
	this.setTitle("GreenStone 3 Administration Tool");
	
	display();
    }

    //private createMenuEvent (){

    public void actionPerformed (ActionEvent event){
	Object esrc = event.getSource();
	// File Options
	if (esrc == menu_bar.file_exit){
	    System.exit(1);
	}	
	//Edit Options
	//Help Options
	else if (esrc == menu_bar.help_conf){
	}
	else if (esrc == menu_bar.help_ext){
	} 
	else if (esrc == menu_bar.help_monitor){
	}
	else if (esrc == menu_bar.help_log){
	}
	else if (esrc == menu_bar.help_about) {
	}
    }

    public void destory (){
	// Destroying create pane ensures the latest log has been closed
	//if (status_pane != null) {
	//}
    }

    // Listens to actions upon the menu bar
    private class MenuListenerImpl
	implements MenuListener {

	public void menuCanceled (MenuEvent e) {
	}
	
	public void menuDeselected (MenuEvent e){
	}
	
	public void menuSelected (MenuEvent e){
	    if (e.getSource() == menu_bar.help){
		if (menu_bar.help.isSelected()){
		    menu_bar.help.doClick(10);
		}
	    }
	}
    }

    public void display(){
	CONF_ICON = new ImageIcon (GAI.admin_path+"images/properties24.gif");
	EXT_ICON = new ImageIcon ("");	
	MONITOR_ICON = new ImageIcon (GAI.admin_path+ "images/examine.gif");
	LOG_ICON = new ImageIcon (GAI.admin_path+"images/collection.gif");
	
	content_pane = (JPanel) this.getContentPane();

	try {
	    this.setSize(size);

	    //create a menuBar
	    menu_bar = new MenuBar();
	    
	    this.setJMenuBar(menu_bar);
	    //Create the tabbed pane and put it in the center
	    tab_pane = new JTabbedPane();
	    tab_pane.addChangeListener(this);
	    
	    //set up the Configuration Pane
	    if (Configuration.get("admin.conf", true)){
		conf_pane = new ConfPane();
		conf_pane.display();
		tab_pane.addTab("Configuration", CONF_ICON, conf_pane);
		tab_pane.setEnabledAt(tab_pane.indexOfComponent(conf_pane),Configuration.get("admin.conf", false));
	    }
	    	    
	    //set up the ext Pane
	    if (Configuration.get("admin.ext", true)){
		ext_pane = new JPanel();
		//ext_pane.display();
		tab_pane.addTab("Exts", EXT_ICON, ext_pane);
		tab_pane.setEnabledAt(tab_pane.indexOfComponent(ext_pane),Configuration.get("admin.ext", false));
	    }
	    	    
	    //set up the Monitor Pane
	    if (Configuration.get("admin.monitor", true)){
		monitor_pane = new JPanel();
		//monitor_pane.display();
		tab_pane.addTab("Monitor", MONITOR_ICON, monitor_pane);
		tab_pane.setEnabledAt(tab_pane.indexOfComponent(monitor_pane),Configuration.get("admin.monitor", false));
	    }

	    //set up the Log Pane
	    if (Configuration.get("admin.log", true)){
		log_pane = new LogPane();
		log_pane.display();
		tab_pane.addTab("Log", LOG_ICON, log_pane);
		tab_pane.setEnabledAt(tab_pane.indexOfComponent(log_pane),Configuration.get("admin.log", false));
	    }   
	    content_pane.setLayout(new BorderLayout());
	    content_pane.add(tab_pane, BorderLayout.CENTER);
	    pack();
	    setVisible(true);
	} catch (Exception e){
	    DebugStream.printStackTrace(e);
	    e.printStackTrace();
	    System.exit(1);
	}
    }

    /** Any implementation of ChangeListener must include this method 
     **so we can be informed when the state of one of the registered 
     **objects changes. In this case we are listening to view changes 
     **within the tabbed pane.
     * @param event A ChangeEvent containing information about the event that fired this call.
     */
    public void stateChanged(ChangeEvent event) {
	if(previous_pane != null) {
	}
	
	//menu_bar.tabSelected(tab_pane.getSelectedIndex());
	int selected_index = tab_pane.getSelectedIndex();
	if (selected_index == tab_pane.indexOfComponent(log_pane)) {
	    log_pane.gainFocus();
	}
	else if (selected_index == tab_pane.indexOfComponent(conf_pane)) {
	    conf_pane.gainFocus();
	}
	previous_pane = (JPanel) tab_pane.getSelectedComponent();
    }
    
    public void exit(){
	DebugStream.println("****exit called");
	System.exit(1);
    }
    
    public void modeChanged (int mode){
	if (log_pane != null){
	    log_pane.modeChanged(mode);
	}
	if (conf_pane !=null) {
	    conf_pane.modeChanged(mode);
	}
    }

    //   public void refresh (int refresh_reason, boolean collection_loaded)
    public void refresh () {
	if (log_pane != null ){
	}
	//if (status_pane != null) {
	//}
    }

    public void returnToInitialPane(){
	if (log_pane != null) {
	    tab_pane.setSelectedComponent(log_pane);
	}
    }
    private class TabUpdater
	implements Runnable {
	private boolean ready = false;
	private int conf_pos = -1;
	private int ext_pos = -1;
	private int monitor_pos = -1;
	private int log_pos = -1;
	private JTabbedPane tab_pane = null;
	
	public TabUpdater (JTabbedPane tab_pane, boolean ready){
	    this.ready = ready;
	    this.tab_pane = tab_pane;
	    conf_pos = tab_pane.indexOfComponent(conf_pane);
	    ext_pos = tab_pane.indexOfComponent(ext_pane);
	    monitor_pos = tab_pane.indexOfComponent(monitor_pane);  
	    log_pos = tab_pane.indexOfComponent(log_pane);
	}
	
	public void run() {
	    if (conf_pos != -1){
		if (ready) {
		    tab_pane.setEnabledAt (conf_pos,Configuration.get("admin.conf", false));
		} else {
		    tab_pane.setEnabledAt (conf_pos,Configuration.get("admin.conf",true));
		}
	    }
	    if (ext_pos != -1){
		if (ready) {
		    tab_pane.setEnabledAt (ext_pos,Configuration.get("admin.ext", false));
		} else {
		    tab_pane.setEnabledAt (ext_pos,Configuration.get("admin.ext",true));
		}
	    }
	    if (monitor_pos != -1){
		if (ready) {
		    tab_pane.setEnabledAt (monitor_pos,Configuration.get("admin.monitor", false));
		} else {
		    tab_pane.setEnabledAt (monitor_pos,Configuration.get("admin.monitor",true));
		}
	    }
	    if (log_pos != -1){
		if (ready) {
		    tab_pane.setEnabledAt (log_pos,Configuration.get("admin.log", false));
		} else {
		    tab_pane.setEnabledAt (log_pos,Configuration.get("admin.log",true));
		}
	    }
	}
	public void setReady (boolean ready){

	    this.ready = ready;
	}
    }
}


